#include <termios.h>

#include <signal.h>

#include <math.h>

#include <stdio.h>

#include <stdlib.h>

#include <ros/ros.h>

#include <geometry_msgs/Twist.h>

// Motor State

#define KEYCODE_P 0x70

// Drive Commands

#define KEYCODE_A 0x61

#define KEYCODE_D 0x64

#define KEYCODE_S 0x73

#define KEYCODE_W 0x77

#define KEYCODE_A_CAP 0x41

#define KEYCODE_D_CAP 0x44

#define KEYCODE_S_CAP 0x53

#define KEYCODE_W_CAP 0x57

class TeleopPioneerKeyboard

{

private:

	double walk_vel_, run_vel_, yaw_rate_, yaw_rate_run_;

	double pan_rate_, tilt_rate_, zoom_rate_;


	geometry_msgs::Twist cmd_vel_;



//nodos

	ros::NodeHandle n_;
	ros::Publisher vel_pub_;

public:

	void init()

	{

		cmd_vel_.linear.x = cmd_vel_.linear.y = cmd_vel_.angular.z = 0;
		vel_pub_ = n_.advertise<geometry_msgs::Twist>("cmd_vel", 1);

		ros::NodeHandle n_private("~");
		n_private.param("walk_vel", walk_vel_, 0.5);
		n_private.param("run_vel", run_vel_, 1.0);
		n_private.param("yaw_rate", yaw_rate_, 1.0);
		n_private.param("yaw_run_rate", yaw_rate_run_, 1.5);

	}

	~TeleopPioneerKeyboard() {
	}

	void keyboardLoop();

};

int kfd = 0;

struct termios cooked, raw;

void quit(int sig)

{

	tcsetattr(kfd, TCSANOW, &cooked);

	exit(0);

}

int main(int argc, char** argv)

{

	ros::init(argc, argv, "pr2_base_keyboard");

	TeleopPioneerKeyboard tpk;

	tpk.init();

	signal(SIGINT, quit);

	tpk.keyboardLoop();

	return (0);

}

void TeleopPioneerKeyboard::keyboardLoop()

{

	char c;



	bool vel_dirty = false;

	// get the console in raw mode

	tcgetattr(kfd, &cooked);

	memcpy(&raw, &cooked, sizeof(struct termios));

	raw.c_lflag &= ~(ICANON | ECHO);

	// Setting a new line, then end of file

	raw.c_cc[VEOL] = 1;

	raw.c_cc[VEOF] = 2;

	tcsetattr(kfd, TCSANOW, &raw);

	puts("Reading from keyboard");

	puts("---------------------------");

	puts("-------ESTOY DENTRO--------");

	puts("---------------------------");

	puts("Use 'WS' to translate");

	puts("Use 'AD' to yaw");

	puts("Use '04' para parar y encender el motor");

	puts("Press 'Shift' to run");

	for (;;)

	{

		// get the next event from the keyboard

		if (read(kfd, &c, 1) < 0)

		{

			perror("read():");

			exit(-1);

		}

		// Clear the previous commands

		cmd_vel_.linear.x = cmd_vel_.linear.y = cmd_vel_.angular.z = 0;
		switch (c)

		{

		case KEYCODE_W:
			cmd_vel_.linear.x = walk_vel_;
			vel_dirty = true;
			break;

		case KEYCODE_S:

			cmd_vel_.linear.x = -walk_vel_;

			vel_dirty = true;

			break;

		case KEYCODE_A:

			cmd_vel_.angular.z = yaw_rate_;

			vel_dirty = true;

			break;

		case KEYCODE_D:

			cmd_vel_.angular.z = -yaw_rate_;

			vel_dirty = true;

			break;

			// Running

		case KEYCODE_W_CAP:

			cmd_vel_.linear.x = run_vel_;

			vel_dirty = true;

			break;

		case KEYCODE_S_CAP:

			cmd_vel_.linear.x = -run_vel_;

			vel_dirty = true;

			break;

		case KEYCODE_A_CAP:

			cmd_vel_.angular.z = yaw_rate_run_;

			vel_dirty = true;

			break;

		case KEYCODE_D_CAP:

			cmd_vel_.angular.z = -yaw_rate_run_;

			vel_dirty = true;

			break;

		}




			vel_pub_.publish(cmd_vel_);

	}

	}
