#include <ros/ros.h>
#include <tf/transform_broadcaster.h>
#include <nav_msgs/Odometry.h>

void poseCallback(const nav_msgs::Odometry::ConstPtr& odomsg) {
	//TF odom=> base_link

	static tf::TransformBroadcaster odom_broadcaster;
	odom_broadcaster.sendTransform(
			tf::StampedTransform(
					tf::Transform(
							tf::Quaternion(odomsg->pose.pose.orientation.x,
									odomsg->pose.pose.orientation.y,
									odomsg->pose.pose.orientation.z,
									odomsg->pose.pose.orientation.w),
							tf::Vector3(odomsg->pose.pose.position.x ,
									odomsg->pose.pose.position.y,
									odomsg->pose.pose.position.z)),
					odomsg->header.stamp, "/odom", "/base_link"));
	ROS_DEBUG("odometry frame sent");
}

int main(int argc, char** argv) {
	ros::init(argc, argv, "pioneer_tf_publisher");
	ros::NodeHandle n;

	ros::Rate r(1000);

	tf::TransformBroadcaster broadcaster;

	//subscribe to pose info
	ros::Subscriber pose_sub = n.subscribe<nav_msgs::Odometry>("RosAria/pose",
			1, poseCallback);

	while (n.ok()) {
		//base_link => laser
		broadcaster.sendTransform(tf::StampedTransform(tf::Transform(tf::Quaternion(0, 0, 0, 1),tf::Vector3(0.15, 0.0, 0.55)),
						ros::Time::now(), "/base_link", "/openni_camera")); ///0.13, -0.04, 0.294

		ros::spinOnce();
		r.sleep();
	}
}

